#include <p18f452.h>
#include "main.h"
#include "afficheur.h"
#include "delay.h"

//Ce fichier va contenir les fonctions permettant de grer l'afficheur


//Tableau contenant l'tat de chaque point, ligne/colonne
//unsigned char pour 8 bits, short int pour 16 bits
unsigned char AfficheurLSB[AFF_NB_LIGNES];
unsigned char AfficheurMSB[AFF_NB_LIGNES];


void AFFConvert_Coord(unsigned char colonneaff, unsigned char ligneaff, unsigned char *colonneuC, unsigned char *ligneuC)
{
	//Cette fonction va permettre de faire la conversion entre les coordonnes
	//au niveau de l'afficheur et celles au niveau du microcontroleur.
	//On numrote en partant de zero l'afficheur

	//On dtermine l'quivalence des lignes
	switch(ligneaff)
	{
		case 0: *ligneuC = 0; break;
		case 1: *ligneuC = 2; break;
		case 2: *ligneuC = 1; break;
		case 3: *ligneuC = 3; break;
		case 4: *ligneuC = 4; break;
		case 5: *ligneuC = 5; break;
		case 6: *ligneuC = 6; break;
		case 7: *ligneuC = 7; break;
		case 8: *ligneuC = 13; break;
		case 9: *ligneuC = 12; break;
		case 10: *ligneuC = 15; break;
		case 11: *ligneuC = 14; break;
		case 12: *ligneuC = 9; break;
		case 13: *ligneuC = 8; break;
		case 14: *ligneuC = 11; break;
		case 15: *ligneuC = 10; break;
		default: *ligneuC = 0;
	}

	//On fait pareil avec les colonnes
	switch(colonneaff)
	{
		case 3: *colonneuC = 1; break;
		case 4: *colonneuC = 0; break;
		case 5: *colonneuC = 8; break;
		case 6: *colonneuC = 6; break;
		case 7: *colonneuC = 5; break;
		case 8: *colonneuC = 4; break;
		case 9: *colonneuC = 7; break;
		case 10: *colonneuC = 3; break;
		case 11: *colonneuC = 2; break;
		default: *colonneuC = 0; break;
	}
}

void AFFSetBit(unsigned char colonne, unsigned char ligne)
{
	//Cette fonction permet de positionne le bit spcifi
	//du tableau
	//Si le bit est entre 0 et 7, on le met en LSB
	//Si 8, en MSB

	unsigned char masque;
	unsigned char ligneconvertie, colonneconvertie;

	//On vient convertir les coordonnes
	AFFConvert_Coord(colonne, ligne, &colonneconvertie, &ligneconvertie);
	
	if (colonneconvertie <8)
	{
		//On calcule le masque pour positionner le bit
		masque = 0x01 << colonneconvertie;

		//On place le bit spcifi a un
		AfficheurLSB[ligneconvertie] = AfficheurLSB[ligneconvertie] | masque;
	}
	else
	{
		//Ne marche que pour ligne = 8. On utilisera << ligne - 8 si il y en a plus
		masque = 0x01;
		AfficheurMSB[ligneconvertie] = AfficheurMSB[ligneconvertie] | masque;
	}
}

void AFFResetBit(unsigned char colonne, unsigned char ligne)
{
	//Cette fonction permet de retirer le bit spcifi
	//du tableau
	//Si le bit est entre 0 et 7, on le met en LSB
	//Si 8, en MSB

	unsigned char masque;
	unsigned char ligneconvertie, colonneconvertie;

	//On vient convertir les coordonnes
	AFFConvert_Coord(colonne, ligne, &colonneconvertie, &ligneconvertie);
	
	if (colonneconvertie <8)
	{
		//On calcule le masque pour positionner le bit
		masque = 0x01 << colonneconvertie;

		//On place le bit spcifi a zero
		AfficheurLSB[ligneconvertie] = AfficheurLSB[ligneconvertie] & ~masque;
	}
	else
	{
		//Ne marche que pour ligne = 8. On utilisera << ligne - 8 si il y en a plus
		masque = 0x01;
		AfficheurMSB[ligneconvertie] = AfficheurMSB[ligneconvertie] & ~masque;
	}
}

void AFFRefresh()
{
	//Cette fonction va permettre de rafraichir l'tat de l'afficheur
	//Elle est active par l'interruption du timer0
	//a chaque appel on passe a la ligne suivante
	//Pour chaque ligne du tableau, on fait:
	
	//On teint les colonnes
	//On active la ligne correspondant a la ligne en cours
	//On vient positionner les xx bits de la ligne sur le port de sortie de la colonne
	//On passe a la ligne suivante

	//Ce procd permet que l'utilisateur ne voit pas le clignotement
	//de l'affichage

	static unsigned char i=0;
	unsigned char masque;

	//Si on est arriv au bout, on rembobine
	if (i==16) i=0;

	masque = i << 4;

	//on eteint
	PORT_LSB_COL_AFF = 0;

	//Bits de poids fort a zero
	PORT_LIGNE_AFFICHEUR = PORT_LIGNE_AFFICHEUR & 0b00001111;

	//On positionne la ligne, 4 bits de poids fort
	PORT_LIGNE_AFFICHEUR = PORT_LIGNE_AFFICHEUR | masque;

	//on valide
	//PORTCbits.RC2 = 1;

	//On positionne les bits colonne lsb correspondant
	PORT_LSB_COL_AFF = AfficheurLSB[i];

	//Idem avec le bit MSB
	if (AfficheurMSB[i]) PORTCbits.RC3 = 1;
	else PORTCbits.RC3 = 0;

	i++;

	//on remet le port c a zero
	//PORTCbits.RC2 = 0;
}

void AFFInit()
{
	//Cette fonction sert  initialiser l'afficheur
	//ainsi que les ports d'e/s

	unsigned char x;

	//On remet le tableau a zero
	AFFReset();

	//On initialise les ports d'e/s
	//RC3 en sortie (0)
	//LSB en sortie 
	TRIS_MSB_COL_AFF = TRIS_MSB_COL_AFF & 0b11110111;
	TRIS_LSB_COL_AFF = 0;

	//Port des lignes en sortie, quatre bits de poids fort
	TRIS_LIGNE_AFFICHEUR = TRIS_LIGNE_AFFICHEUR & 0b00001111;

	//rc2 en sortie
	TRISC = TRISC & 0b11111011;

	//on active rc2
	PORTCbits.RC2 = 1;
}

void AFFReset(void)
{
	//Cette fonction va permettre de remettre l'afficheur a zero

	unsigned char x;

	for(x=0; x<=AFF_NB_LIGNES;x++)
	{
		AfficheurLSB[x] = 0;
		AfficheurMSB[x] = 0;
	}
}

void AFFNbPistes(unsigned char PisteMin, unsigned char PisteMax)
{
	//Cette fonction va permettre d'afficher le nombre de pistes
	//Du cd sur l'afficheur

	//Si il y a 15 pistes ou plus, on affiche le symbole correspondant
	unsigned char i;

	for(i=1; i<16;i++)
	{
		//Si la piste est a allumer
		if ( (i >= PisteMin) && (i <= PisteMax))
			AFFSetBit(11,i-1);
		else
			AFFResetBit(11,i-1);
	}

	//Si 15+
	if (PisteMax > 15) AFFSetBit(11,15);
	else AFFResetBit(12,15);
}

void AFFChiffre(unsigned char chiffre, unsigned char colonne)
{
	//Cette fonction permet d'afficher le chiffre slectionn
	//Sur la colonne choisie.

	unsigned char i;

	//On remet le chiffre a zero
	for (i=0; i<=15;i++)
	{
		AFFResetBit(colonne,i);
	}

	//On allume les segments du chiffre
	switch(chiffre)
	{
		case 0:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_F);
			break;
		}

		case 1:
		{
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			break;
		}

		case 2:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			break;
		}

		case 3:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 4:
		{
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 5:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_D);
			break;
		}

		case 6:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_G);
			break;
		}
		
		case 7:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			break;
		}

		case 8:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 9:
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}
	}
}

void AFFNombre(unsigned char nombre, unsigned char colonne)
{
	//Cette fonction va permettre d'afficher un nombre
	//Sur l'afficheur a l'emplacement spcifi
	//Si le nombre est < 10, on rajoute un zero devant

	unsigned char dizaines, unites;

	if (nombre < 10)
	{
		AFFChiffre(0, colonne);
		unites = nombre;
	}
	else
	{
		//On affiche le chiffre des dizaines
		dizaines = (unsigned char)nombre / 10;
		AFFChiffre(dizaines, colonne);

		nombre = nombre - (dizaines *10);
	}

	//On affiche les units
	AFFChiffre(unites, colonne +1);
}

void AFFTemps(unsigned char minutes, unsigned char secondes)
{
	//Cette fonction va permettre d'afficher le temps
	//Sur l'afficheur, en colonnes 7,8,9,10
	//on allume aussi le ":"

	//On allume le ":"
	AFFSetBit(9, 0);

	//on affiche les minutes sur les colonnes 7,8
	AFFNombre(minutes,7);

	//on affiche les units sur les colonnes 9,10
	AFFNombre(minutes, 9);
}

void AFFSymboleLecture(void)
{
	//Cette fonction va permettre d'afficher le symbole de la lecture

	unsigned char i;

	//On commence par remettre les symboles a zero
	for(i=3; i<=15;i++)
	{
		AFFResetBit(3, i);
	}

	//On affiche le symbole
	AFFSetBit(3,3);
	AFFSetBit(3,4);
	AFFSetBit(3,6);
	AFFSetBit(3,7);
	AFFSetBit(3,9);
}

void AFFCaractere(unsigned char colonne, unsigned char caractere)
{
	//Cette fonction va permettre d'afficher un caractre alphanumrique
	//a la colonne spcifie
	
	unsigned char i;

	//On commence par effacer la colonne spcifie
	for (i=0; i <= 15; i++)
	{
		AFFResetBit(colonne, i);
	}

	//On affiche le caractre
	switch (caractere)
	{
		case 'A':
		{
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 'B':
		{
			AFFChiffre(8, colonne);
			break;
		}

		case 'C':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			break;
		}

		case 'D':
		{
			//On l'affiche en minuscule
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_E);
			break;
		}

		case 'E':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 'F':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_G);
			break;
		}

		case 'G':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_H);
			break;
		}

		case 'H':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			break;
		}

		case 'I':
		{
			AFFSetBit(colonne, SEGMENT_MK);
			break;
		}

		case 'J':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_MK);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_E);
			break;
		}

		case 'K':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_N);
			AFFSetBit(colonne, SEGMENT_J);
			break;
		}

		case 'L':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			break;
		}
	
		case 'M':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_L);
			AFFSetBit(colonne, SEGMENT_N);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_C);
			break;
		}

		case 'N':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_L);
			AFFSetBit(colonne, SEGMENT_J);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_B);
			break;
		}

		case 'O':
		{
			AFFChiffre(0,colonne);
			break;
		}

		case 'P':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_B);
			break;
		}

		case 'Q':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_J);
			break;
		}

		case 'R':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_B);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_H);
			AFFSetBit(colonne, SEGMENT_G);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_J);
			break;
		}

		case 'S':
		{
			AFFChiffre(5, colonne);
			break;
		}

		case 'T':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_MK);
			break;
		}

		case 'U':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_B);
			break;
		}

		case 'V':
		{
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_B);
			break;
		}

		case 'W':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_F);
			AFFSetBit(colonne, SEGMENT_E);
			AFFSetBit(colonne, SEGMENT_D);
			AFFSetBit(colonne, SEGMENT_MK);
			AFFSetBit(colonne, SEGMENT_C);
			AFFSetBit(colonne, SEGMENT_B);
			break;
		}

		case 'X':
		{
			AFFSetBit(colonne, SEGMENT_L);
			AFFSetBit(colonne, SEGMENT_N);
			AFFSetBit(colonne, SEGMENT_I);
			AFFSetBit(colonne, SEGMENT_J);
			break;
		}

		case 'Y':
		{
			AFFSetBit(colonne, SEGMENT_L);
			AFFSetBit(colonne, SEGMENT_N);
			AFFSetBit(colonne, SEGMENT_MK);
			break;
		}

		case 'Z':
		{
			AFFSetBit(colonne, SEGMENT_A);
			AFFSetBit(colonne, SEGMENT_N);
			AFFSetBit(colonne, SEGMENT_I);
			AFFSetBit(colonne, SEGMENT_D);
			break;
		}
	}
}

void AFFText(unsigned char text[8])
{
	unsigned char NumCol = 4;

	//Cette fonction va afficher jusqu'a sept lettres sur l'afficheur
	while(text[NumCol-4] != 0)
	{
		AFFCaractere(NumCol,text[NumCol - 4]);
		NumCol++;
	}
}

void AFFSymbolePause(void)
{
	//Cette fonction permet d'afficher le symbole pause a gauche

	AFFSetBit(3,3);
	AFFSetBit(3,4);
	AFFSetBit(3,5);
	AFFSetBit(3,6);
	AFFSetBit(3,7);
	AFFSetBit(3,8);
	AFFSetBit(3,11);
	AFFSetBit(3,12);
	AFFSetBit(3,13);
	AFFSetBit(3,14);
	AFFSetBit(3,15);
}
