#include <p18f452.h>
#include "main.h"
#include "dsa.h"
#include "cd.h"
#include "delay.h"
#include "afficheur.h"

struct TState Etat;
struct TTrackInfo TrackInfo[99];	//Informations sur les pistes
unsigned char ModeAffichageTemps;	//mode d'affichage du temps

//Ce fichier va permettre de grer toutes les fonctions de lecture
//du cd, tels que play, pause, prev, next, etc...


void CDInit()
{
	//Cette fonction va permettre d'initialiser toutes les variables
	//lors de la mise en marche du microcontroleur

	//on reset les variables
	CDReset();

	//On met la platine en tat arrt
	ALIM_220V = 0;
	ALIM_NV1 = 0;
	ALIM_NV2 = 0;
	ALIM_RELAIS_SORTIE = 0;

	//on place les quatre lignes des relais en sortie
	TRIS_ALIM_220V = TRIS_ALIM_220V & ~MSQ_ALIM_220V;
	TRIS_ALIM_NV1 = TRIS_ALIM_NV1 & ~MSQ_ALIM_NV1;
	TRIS_ALIM_NV2 = TRIS_ALIM_NV2 & ~MSQ_ALIM_NV2;
	TRIS_ALIM_RELAIS_SORTIE = TRIS_ALIM_RELAIS_SORTIE & ~MSQ_ALIM_RELAIS_SORTIE;

	//On met les quatre relais a zero
	ALIM_220V = 0;
	ALIM_NV1 = 0;
	ALIM_NV2 = 0;
	ALIM_RELAIS_SORTIE = 0;

	//On place les lignes du tiroir en sortie
	TRIS_TIROIR_EN = TRIS_TIROIR_EN & ~MSQ_TIROIR_EN;
	TRIS_TIROIR_1A = TRIS_TIROIR_1A & ~MSQ_TIROIR_1A;
	TRIS_TIROIR_2A = TRIS_TIROIR_2A & ~MSQ_TIROIR_2A;

	//On place le tiroir en sortie, dsactiv
	TIROIR_EN = 0;
	TIROIR_1A = 0;
	TIROIR_2A = 1;

	//On configure les capteurs FDC en entre
	TRIS_CAPTEUR_FDC_E = TRIS_CAPTEUR_FDC_E | MSQ_CAPTEUR_FDC_E;
	TRIS_CAPTEUR_FDC_S = TRIS_CAPTEUR_FDC_S | MSQ_CAPTEUR_FDC_S;
}

void CDMarche(void)
{
	//Cette fonction va permettre de mettre en marche le lecteur CD
	//En respectant les delais et l'ordre de la prsence des alimentations

	//On vrifie que le lecteur est bien teint
	if (!Etat.CdPowered)
	{
		//On reset toutes les variables
		CDReset();

		//On place le 220v
		ALIM_220V = 1;

		//On attend deux secondes
		DelayS(2);

		//On place l'alimentation de niveau 1 (horloge, 5v mca, afficheur)
		ALIM_NV1 = 1;

		//On attend une seconde
		DelayS(1);
		
		//On place l'alimentation de niveau 2
		ALIM_NV2 = 1;

		//On attend une seconde
		DelayS(1);
		
		//On colle le relais de sortie ligne
		ALIM_RELAIS_SORTIE = 1;

		//La platine est allume
		Etat.CdPowered = 1;
	}
}

void CDArret(void)
{
	//Cette fonction va permettre d'teindre proprement le lecteur cd
	
	//Si il est allum
	if (Etat.CdPowered)
	{
		//On vrifie que le CD ne tourne pas
		if (Etat.CdPlaying) CDStop();

		//On coupe les relais de sortie
		ALIM_RELAIS_SORTIE = 0;
		
		//on coupe l'alim nv2
		ALIM_NV2 = 0;

		//on attend une seconde
		DelayS(1);

		//On coupe l'alim niveau 1
		ALIM_NV1 = 0;

		//On attend une seconde
		DelayS(1);

		//On coupe l'alim 220v
		ALIM_220V = 0;

		//le lecteur est teint
		Etat.CdPowered = 0;
	}
}

void CDLecture(unsigned char track)
{
	//Cette fonction permet de mettre la platine en lecture
	//Il faut que la TOC ait t lue
	//Et que le CD ne tourne pas

	if (Etat.TocReaded & !Etat.CdPlaying)
	{
		//On envoi la commande de lecture
		DSASend(PLAY_TITLE, track);
	}
}

void CDStop(void)
{
	//Cette fonction permet d'arrter la lecture de la platine

	//Si la platine est en marche
	if (Etat.CdPowered)
	{
		//Si le cd est sorti
		if (Etat.CdEjected) CDRentrerTiroir();

		DSASend(STOP, 0x00);

		//On attend qu'il s'arrete
		while(Etat.CdPlaying) {}
	}
}

void CDReadTOC(void)
{
	//Cette fonction permet de lire la TOC d'un cd
	
	//Si la platine est en marche
	if (Etat.CdPowered)
	{
		DSASend(READ_TOC, 0x00);

		//On marque la toc comme lue
		Etat.TocReaded = 1;
	}
}

void CDClearTOC(void)
{
	unsigned char i;
	//Cette fonction permet d'annuler la TOC actuelle

	//on envoie la commande clear_toc
	DSASend(CLEAR_TOC, 0);

	//On annule la toc
	Etat.TocReaded = 0;

	//On reset les infos des pistes
	Etat.TrackNumber = 0;
	Etat.CdLengthMin = 0;
	Etat.CdLengthSec = 0;
	Etat.CurrentTrack = 0;

	for(i=0; i<=99;i++)
	{
		TrackInfo[i].TimeMin = 0;
		TrackInfo[i].TimeSec = 0;
	}
}

void CDTraitementDSA(unsigned char code, unsigned char argument)
{
	//Cette fonction va servir  traiter les vnements recus depuis
	//l'interface DSA

	//On analyse le code
	switch(code)
	{
		case ACTUAL_TITLE: 
		{
			//On change le numro de la piste en cours
			Etat.CurrentTrack = argument;
			break;
		}

		case ACTUAL_REL_MINUTES:
		{
			//On vient mettre a jour les minutes relatives
			TrackInfo[Etat.CurrentTrack].TimeMin = argument;
			break;
		}

		case ACTUAL_REL_SECONDS:
		{
			//On vient mettre a jour le temps en secondes
			TrackInfo[Etat.CurrentTrack].TimeSec = argument;
			break;
		}

		case STOPPED:
		{
			//On indique que le CD est arrte
			Etat.CdPlaying = 0;
			break;
		}

		case TOC_MAX_TRACK:
		{
			//On vient remplacer le nombre de pistes du disque
			Etat.TrackNumber = argument;
			break;
		}

		case DISC_TIME_MIN:
		{
			//On vient mettre a jour le nombre de minutes totale du disque
			Etat.CdLengthMin = argument;
			break;
		}

		case DISC_TIME_SEC:
		{
			//On vient mettre a jour le nombre de secondes totale du disque
			Etat.CdLengthSec = argument;
			break;
		}
	}
}

void CDSortirTiroir(void)
{
	//Cette fonction va permettre de sortir le tiroir du lecteur CD
	//Elle s'arrte lorsqu'on atteint le capteur fin de course

	if (!Etat.CdEjected)
	{
		//on vient mettre le moteur en marche
		TIROIR_1A = 0;
		TIROIR_2A = 1;

		TIROIR_EN = 1;

		//On attend le capteur fin de course S (sortie)
		while(CAPTEUR_FDC_S) { }

		//On arrte le tiroir
		TIROIR_EN = 0;

		//le cd est eject
		Etat.CdEjected = 1;

		//On reset la TOC
		CDClearTOC();
	}	
}

void CDRentrerTiroir(void)
{
	//Cette fonction va faire le contraire de la prcdente.

	if(Etat.CdEjected)
	{
		//on met le moteur en marche, en sens inverse
		TIROIR_1A = 1;
		TIROIR_2A = 0;

		TIROIR_EN = 1;

		//on attend le capteur E
		while(CAPTEUR_FDC_E) { }

		//on arrte le tiroir
		TIROIR_EN = 0;

		//le cd est rentr
		Etat.CdEjected = 0;

		//On vient lire la TOC
		CDReadTOC();
	}
}

//Interface pour le clavier et la tlcommande

void CDMarcheArret(void)
{
	//Si le CD est arrt, on met en marche
	//sinon on teint

	if (Etat.CdPowered) CDArret();
	else CDMarche();
}

void CDPlayStop(void)
{
	//Si le CD est en lecture, on l'arrte et vice-versa
	
	if (Etat.CdPlaying)	CDStop;
	else CDLecture(1);
}

void CDEject(void)
{
	//Si le tiroir est sorti, on le rentre et vice-versa

	if (Etat.CdEjected)
	{
		//On rentre le tiroir
		CDRentrerTiroir();
	}
	else
	{
		//On s'assure que le CD est arrt avant
		if (Etat.CdPlaying)
		{
			CDStop();
		}
		CDSortirTiroir();
	}
}

void CDRefresh(void)
{
	//Cette fonction va permettre de grer tout ce qui doit tre remis  jour
	//priodiquement, tel que les informations afficheur

	//on efface l'afficheur
	AFFReset();

	//On affiche le temps de la piste si le cd est en lecture
	if (Etat.CdPlaying)
	{
		AFFSymboleLecture();

		//Selon le mode d'affichage du temps
		if (ModeAffichageTemps == AFF_TEMPS_PISTE_ECOULE)
		{
			AFFTemps(TrackInfo[Etat.CurrentTrack].TimeMin,TrackInfo[Etat.CurrentTrack].TimeMin);
		}
	}

	//Si le cd est en pause, on affiche le symbole pause
	if (Etat.CdPaused)
	{
		AFFSymbolePause();
	}

}

void CDReset(void)
{
	unsigned char i;
	//On remet toutes les variables a zero
	Etat.TrackNumber = 0;
	Etat.CdLengthMin = 0;
	Etat.CdLengthSec = 0;

	Etat.CurrentTrack = 0;
	for(i=0; i<=99;i++)
	{
		TrackInfo[i].TimeMin = 0;
		TrackInfo[i].TimeSec = 0;
	}

	Etat.CdPowered = 0;
	Etat.CdPlaying = 0;
	Etat.CdPaused = 0;
	Etat.TocReaded = 0;
	Etat.CdPresent = 0;
	Etat.CdEjected = 0;

	//mode d'affichage du temps
	ModeAffichageTemps = AFF_TEMPS_PISTE_ECOULE;
}

void CDPause(void)
{
	//Cette fonction permet de mettre/retirer la pause

	if (Etat.CdPaused)
	{
		//On envoi la commande pour relacher la pause
		DSASend(PAUSE_RELEASE,0);
		Etat.CdPaused = 0;
	}
	else
	{
		//On met en pause
		DSASend(PAUSE,0);
		Etat.CdPaused = 1;
	}
}
