//Ce fichier va contenir toutes les commandes spcifiques
//a l'interface de communication DSA entre le microcontroleur
//et le drive

#include <p18f452.h>
#include "main.h"
#include "dsa.h"

void DSAInit(void)
{
	//cette fonction va permettre d'initialiser le bus dsa
	//lors de la mise en route du micro controleur

	//On met le port en entre
	DSAConfigIn();
}

void DSAConfigIn(void)
{
	//Cette fonction permet de placer les e/s DSA en tat d'attente
	//On place DATA en entre, STROBE en entre et ACK en sortie

	PORT_DSA_DATA = PORT_DSA_DATA | MSQ_DSA_DATA;
	PORT_DSA_STROBE = PORT_DSA_STROBE | MSQ_DSA_STROBE;
	PORT_DSA_ACK = PORT_DSA_ACK & ~MSQ_DSA_ACK;
}

void DSAConfigOut(void)
{
	//Cette fonction permet de placer le port DSA en sortie
	//On place data en sortie, strobe en sortie, ack en entre

	PORT_DSA_DATA = PORT_DSA_DATA & ~MSQ_DSA_DATA;
	PORT_DSA_STROBE = PORT_DSA_STROBE & ~MSQ_DSA_STROBE;
	PORT_DSA_ACK = PORT_DSA_ACK | MSQ_DSA_ACK;
}

void DSASendSynchro(void)
{
	//Cette fonction va prendre en charge l'envoi de la synchronisation
	//du protocole DSA

	//On place un niveau bas sur DSA
	DSA_DATA = 0;

	//On attend un niveau bas sur ACK
	while (DSA_ACK)
	{
	}

	//quand on l'a, on remet data au niveau haut
	DSA_DATA = 1;

	//On attend que ack revienne a un
	while(!DSA_ACK)
	{
	}
	
	//C'est fini, on doit tre synchro !
}

void DSAReceiveSynchro(void)
{
	//Cette fonction permet de recevoir une demande de synchronisation
	//On attend que data passe a zero
	//On positionne ensuite ACK
	//On attend que data revienne a un
	//On remet ACK a un

	//On attend que data soit a zero
	while(!DSA_DATA) {}

	//On met ACK a zero
	DSA_ACK = 0;

	//On attend que data revienne a un
	while(DSA_DATA) {}

	//On remet ACK a un
	DSA_ACK = 1;
}

void DSAReceiveAck(void)
{
	//Cette fonction va prendre en charge l'acquittement de la transmission par le recepteur
	//L'emetteur place ACK a zero
	//Le recepteur place data a zero si la transmission est invalide
	// puis place strobe a zero
	
	unsigned char ValidData;

	//On place ack a zero
	DSA_ACK = 0;

	//On attend que strobe passe a zero
	while(DSA_STROBE) {}

	//On vient lire data
	ValidData = !DSA_DATA;

	//on remet ACK a un
	DSA_ACK = 1;

	//On attend que strobe remonte
	while(!DSA_STROBE) {}
}

void DSASendAck(unsigned char ok)
{
	//Cette fonction permet d'envoyer l'acquittement de la reception des donnees
	//On attend que l'emetteur place ACK a zero
	//On positionne data a zero si la transmission est ok, un sinon
	//On place strobe a zero
	//On attend que ACK repasse a un
	//On replace strobe a un

	//On attend que ACK passe a zero
	while(!DSA_ACK) {}

	//On positionne DATA
	DSA_DATA = ok ? 0:1;

	//On place strobe a zero
	DSA_STROBE = 0;

	//On attend que ACK remonte
	while(!DSA_ACK) {}

	//On relache strobe
	DSA_STROBE = 1;
}

void DSASendChar(unsigned char commande)
{
	//Cette fonction envoie un octet sur un bus DSA

	unsigned char i;

	//on envoie les 8 bits de commande
	for(i=0; i<8;i++)
	{
		//On envoie le bit numero i
		
		//On positionne le bit sur data
		DSA_DATA = (commande & 0b10000000) ? 1:0;

		//On dcale commande vers la gauche
		commande = commande << 1;

		//On met strobe au niveau bas
		DSA_STROBE = 0;

		//On attend que ack passe au niveau bas
		while(DSA_ACK)
		{
		}
		
		//On remet strobe au niveau haut
		DSA_STROBE = 1;

		//On attend que ack repasse au niveau haut
		while(!DSA_ACK)
		{
		}
		
		//On passe au bit suivant
	}
}

unsigned char DSAReceiveChar(void)
{
	unsigned char BitCount;
	unsigned char Data;

	Data = 0;

	//Cette fonction va permettre de recevoir 8 bits depuis le bus DSA
	for (BitCount = 0; BitCount < 8; BitCount++)
	{
		//On attend que strobe passe a zero
		while(DSA_STROBE) {}

		//On dcale a gauche
		Data = Data << 1;

		//On vient lire data
		Data += DSA_DATA;

		//On met ack a zero
		DSA_ACK = 0;

		//On attend que strobe remonte
		while (!DSA_STROBE) {}
		
		//on remet ack a un
		DSA_ACK = 1;
	}
}

void DSASend(unsigned char commande, unsigned char argument)
{
	//Cette fonction va permettre d'envoyer la commande
	//et l'argument passs en paramtre sur un bus DSA

	unsigned char i;
	
	//On place le port en sortie
	DSAConfigOut();

	//On se synchronise
	DSASendSynchro();
	
	//On envoie la commande
	DSASendChar(commande);

	//On envoie l'argument
	DSASendChar(argument);

	//On repasse le port en entre
	DSAConfigIn();
	
	//On verifie la transmission
	DSAReceiveAck();
}

void DSAReceive(unsigned char *commande, unsigned char *argument)
{
	//Cette fonction va permettre d'effectuer la reception d'une commande DSA

	//On place le bus en entre
	DSAConfigIn();

	//On attend une demande de synchronisation
	DSAReceiveSynchro();
	
	//On recoit un caractere
	*commande = DSAReceiveChar();

	//On recoit un caractere
	*commande = DSAReceiveChar();
	
	//On place le bus en sortie
	DSAConfigOut();

	//On envoi l'acquittement
	DSASendAck(1);

	//On remet le bus en entre
	DSAConfigIn();
}
