#include <p18f452.h>
#include "interrupt.h"
#include "afficheur.h"
#include "rc5.h"
#include "dsa.h"
#include "cd.h"
#include "delays.h"


//Ce fichier va permettre de grer les routines d'interruption

//Nombre d'interruptions en timer1
unsigned char Tmr1NbInterrupts;
extern unsigned char DelayCounter1;

#pragma interrupt process_high_ISR save=DelayCounter1
#pragma interruptlow process_low_ISR save=DelayCounter1

#pragma code low_vector = 0x18
void low_ISR(void)
{
	_asm goto process_low_ISR _endasm
}
#pragma code

#pragma code high_vector = 0x08
void high_ISR(void)
{
	_asm goto process_high_ISR _endasm
}
#pragma code

void process_high_ISR(void)
{
//	unsigned char DSA_code;
//	unsigned char DSA_commande;

	//On cherche la source de l'interruption

	//C'est le timer 0 qui a dclench l'interruption
	if ((INTCONbits.TMR0IE) && (INTCONbits.TMR0IF))
	{
		T0CONbits.TMR0ON = 0;
	
		//On met l'afficheur a jour
		AFFRefresh();

		//on reset l'interruption
		T0CONbits.TMR0ON = 1;
		INTCONbits.TMR0IF = 0;
	}

	//C'est une interruption sur RB0
	if ((INTCONbits.INT0IE) && (INTCONbits.INT0IF))
	{
		//On desactive le timer 0
		INTTimer0Enable(0);

		//On fait la reception RC5
		RC5Reception();

		//On reactive le timer0
		INTTimer0Enable(1);

		//On annule l'interruption
		INTCONbits.INT0IF = 0;
	}

	//Interruption sur timer1
	if(PIR1bits.TMR1IF)
	{
		//On compte une interruption supplmentaire, si on est pas a fond
		if (Tmr1NbInterrupts != 255) Tmr1NbInterrupts++;

		//On recharge le timer
		TMR1H = TMR1_DUREE_H;
		TMR1L = TMR1_DUREE_L;

		//On reset l'interruption
		PIR1bits.TMR1IF = 0;
	}

	//Interruption sur rb2
/*	if(INTCON3bits.INT2IF)
	{
		//On vient faire la reception DSA
		DSAReceive(&DSA_code, &DSA_commande);
	
		//traitement DSA
		CDTraitementDSA(DSA_code, DSA_commande);
	}*/
}

void process_low_ISR(void)
{
	//On cherche la source de l'interruption
}

void INTInit(void)
{
	//Cette fonction sert a initialiser les interruptions

	//interruptions hautes/basses actives
	RCONbits.IPEN = 1;

	//On charge le timer 0
	//TMR0H = TMR0_DUREE_H;
	TMR0L = TMR0_DUREE_L;

	//On l'active
	T0CONbits.T0CS = 0;							//Horloge interne
	T0CONbits.T0SE = 0;							//Front montant
	T0CONbits.PSA = 0;							//Utilisation du prescaler
	T0CONbits.T0PS2 = 1;						//prescaler
	T0CONbits.T0PS1 = 0;						//prescaler
	T0CONbits.T0PS0 = 0;						//prescaler
	INTCON2bits.TMR0IP = 1;						//priorit haute
	T0CONbits.T08BIT = 1;						//Timer en mode 8 bits
	T0CONbits.TMR0ON = 1;
	INTCONbits.TMR0IE = 1;

	//on configure le timer1
	T1CONbits.T1OSCEN = 0;
	T1CONbits.TMR1CS = 0;
	T1CONbits.RD16 = 1;
	IPR1bits.TMR1IP = 1;

	//Timer2
	T2CONbits.TMR2ON = 0;
	PIE1bits.TMR2IE = 0;
	IPR1bits.TMR2IP = 0;

	//Timer3
	T3CONbits.TMR3ON = 0;
	T3CONbits.TMR3CS = 0;		//Horloge interne
	PIE2bits.TMR3IE = 0;
	IPR2bits.TMR3IP = 0;

	//INT0
	INTCONbits.INT0IE = 1;
	INTCON2bits.INTEDG0 = 0;	//Interruption sur front descendant
	INTCON2bits.RBIP = 1;

	//INT1
	INTCON2bits.INTEDG1 = 0;	//Front descendant
	INTCON3bits.INT1IE = 0;
	INTCON3bits.INT1IP = 0;

	//INT2
	INTCON2bits.INTEDG2 = 0;
	INTCON3bits.INT2IP = 1;		//haut niveau
	INTCON3bits.INT2IE = 0;

	//Le reste
	PIE1bits.PSPIE = 0;
	PIE1bits.ADIE = 0;
	PIE1bits.RCIE = 0;
	PIE1bits.TXIE = 0;
	PIE1bits.SSPIE = 0;
	PIE1bits.CCP1IE = 0;
	PIE1bits.TMR2IE = 0;
	PIE1bits.TMR1IE = 0;
	PIE2bits.EEIE = 0;
	PIE2bits.BCLIE = 0;
	PIE2bits.LVDIE = 0;
	PIE2bits.TMR3IE = 0;
	PIE2bits.CCP2IE = 0;
	

	//On reset les interruptions avant d'activer !
	INTCONbits.INT0IF = 0;

	//on active les interruptions
	INTCONbits.GIEH = 1;
	INTCONbits.GIEL = 0;
}

void INTTimer0Enable(unsigned char i)
{
	//Cette fonction permet d'activer/desactiver le timer0
	//Ele permet de faire la reception RC5 sans se soucier 
	//de l'afficheur

	if (i) INTCONbits.TMR0IE = 1;
	else INTCONbits.TMR0IE = 0;
}

void INTTimer1Enable(unsigned char i)
{
	//Cette fonction va activer le timer1
	//selon la valeur de i
	//Elle permet de mesurer la dure d'appui d'une touche du clavier

	if (i)
	{
		//on reset le nombre d'interruptions
		Tmr1NbInterrupts = 0;

		//On charge le timer
		TMR1H = TMR1_DUREE_H;
		TMR1L = TMR1_DUREE_L;

		//on active l'interruption
		PIE1bits.TMR1IE = 1;

		//on lance le timer
		T1CONbits.TMR1ON = 1;
	}
	else
	{
		//on arrete le timer
		T1CONbits.TMR1ON = 0;
		PIE1bits.TMR1IE = 0;
	}
}

unsigned char INTTmr1NbInterrupts(void)
{
	//Cette fonction renvoie le nombre d'interruptions
	//qui ont eu lieu sur le timer1

	return Tmr1NbInterrupts;
}

void INTRB2Enable(unsigned char i)
{
	//Cette fonction permet d'activer/desactiver les interruptions sur RB1
	//Elle est utilise pour dtecter de l'activit sur le port DSA.
	//On la desactive pendant l'envoi et la reception DSA

	if (i)
	{
		//activation de l'interruption
		INTCON3bits.INT2IE = 1;
	}
	else
	{
		INTCON3bits.INT2IE = 0;
	}
}
