#include <p18f452.h>
#include "main.h"
#include "dsa.h"
#include "afficheur.h"
#include "cd.h"
#include "clavier.h"
#include "rc5.h"
#include "interrupt.h"
#include "delay.h"

//Configuration
#pragma config OSC=HS		//mode high speed
#pragma config OSCS=ON		//on active l'oscillateur interne
#pragma config PWRT=ON		//timer mise sous tension
#pragma config BOR=OFF		//brown out reset
#pragma config BORV=25		//tension de brown out
#pragma config WDT=OFF		//watchdog
#pragma config WDTPS=1		//prescaler watchdog
#pragma config CCP2MUX=OFF	//CCP2MUX ?
#pragma config LVP=OFF
#pragma config STVR=OFF		//stack overflow reset
#pragma config DEBUG=OFF

#pragma config CP0=OFF		//protection du code
#pragma config CP1=OFF
#pragma config CP2=OFF
#pragma config CP3=OFF
#pragma config CPB=OFF



void main(void)
{
	unsigned char CommandeDSA, ArgumentDSA;
	unsigned char i,masque;

	//Initialisation
	Initialisation();

	DelayS(1);

	while(1)
	{
		//on regarde si une touche clavier a t presse
		ClavierTraitement();

		//On fait le traitement RC5
		RC5Traitement();

		//On fait la maj des fonctions cd
		//CDRefresh();
	}
}

void Initialisation(void)
{
	//Cette fonction permet de configurer tous les ports d'e/s
	//ainsi que les registres gnraux du pic


	//Configuration INTCON
	INTCONbits.GIEH = 0;						//Interruptions hautes desactivees
	INTCONbits.GIEL = 0;						//Interruptions basses desactivees
	INTCONbits.TMR0IE = 0;						//Interruption du timer0 desactiv
	INTCONbits.INT0IE = 0;						//Interruption pin rb0
	INTCONbits.RBIE = 0;						//Interruption port B
	INTCONbits.TMR0IF = 0;						//Interruption flag timer0
	INTCONbits.INT0IF = 0;						//Interruption flag rb0
	INTCONbits.RBIF = 0;						//Interruption flag port b

	//Configuration INTCON2
	INTCON2bits.RBPU = 1;						//Rsistances de pull-up portb
	INTCON2bits.INTEDG0 = 1;					//Interruption sur front montant int0
	INTCON2bits.INTEDG1 = 1;					//Interruption sur front montant int1
	INTCON2bits.INTEDG2 = 1;					//Interruption sur front montant int2
	INTCON2bits.TMR0IP = 1;						//Priorit de l'interruption tmr0 = high
	INTCON2bits.RBIP = 1;						//Priorit de l'interruption portb

	//Configuration INTCON3
	INTCON3bits.INT2IP = 0;						//Priorit int2
	INTCON3bits.INT1IP = 0;						//Priorit int1
	INTCON3bits.INT2IE = 0;						//Interruption externe int2
	INTCON3bits.INT1IE = 0;						//Interruption externe int1
	INTCON3bits.INT2IF = 0;						//Interruption flag externe 2
	INTCON3bits.INT1IF = 0;						//Interruption flag externe 1

	//Configuration ADCON0
	ADCON0bits.ADCS1 = 0;						//Selection de la frequence de conversion A/D
	ADCON0bits.ADCS0 = 0;						//
	ADCON0bits.CHS0 = 0;						//Selection du canal pour la conversion
	ADCON0bits.CHS1 = 0;
	ADCON0bits.CHS2 = 0;
	ADCON0bits.ADON = 0;						//Activation du module de conversion A/D

	//Configuration ADCON1
	ADCON1bits.ADFM = 0;						//Format de la conversion A/D
	ADCON1bits.ADCS2 = 0;						//Selection de la frquence de conversion
	ADCON1bits.PCFG0 = 0;						//Configuration port AD
	ADCON1bits.PCFG1 = 1;
	ADCON1bits.PCFG2 = 1;
	ADCON1bits.PCFG3 = 0;

	//On configure le timer
	T0CONbits.TMR0ON = 0;						//Active le timer0
	T0CONbits.T08BIT = 0;						//Timer en mode 16 bits
	T0CONbits.T0CS = 0;							//Horloge interne
	T0CONbits.T0SE = 0;							//Front montant
	T0CONbits.PSA = 0;							//Utilisation du prescaler
	T0CONbits.T0PS2 = 1;						//prescaler
	T0CONbits.T0PS1 = 1;						//prescaler
	T0CONbits.T0PS0 = 1;						//prescaler

	//On configure RCON
	RCONbits.IPEN = 1;							//On active les priorits d'interruptions

	//on desactive le lvd
	LVDCONbits.LVDEN = 0;
	

	//Initialisation des diffrents modules
	CDInit();
	DSAInit();
	AFFInit();
	ClavierInit();
	RC5Init();
	INTInit();									//Doit etre le dernier
}
