#include <p18f452.h>
#include "main.h"
#include "rc5.h"
#include "delay.h"
#include "cd.h"
#include "afficheur.h"
#include "eeprom.h"

//Ce fichier va contenir les routines de rception rc5
//et d'auto apprentissage de la tlcommande

//Variables globales
unsigned char commande;			//Code commande
unsigned char code;				//Code appareil
unsigned char repeat;			//Bit de rptition

unsigned char code_telecommande;	//Code appris par auto-apprentissage

extern struct Etat;

void RC5Reception(void)
{
	//Cette fonction va effectuer la rception rc5
	//elle renseigne le code appareil et le code commande recu

	unsigned char NbBits;

	//On met les codes a zero
	code = 0;
	commande = 0;
	repeat = 0;

	//On attend que le signal passe a zero (IR a un)
	while(!RC5_IN) {}

	//Second bit de start
	//on attend le prochain front
	RC5WaitBit(!RC5_IN);
	//On attend 1332uS, 3/4 bit
	Delay1332US();
	
	//on lit la rptition
	if(RC5_IN) repeat = 1;
	RC5WaitBit(!RC5_IN);
	Delay1332US();
	
	//on recoit les 5 bits du code
	NbBits = 5;
	while(NbBits)
	{
		//On dcale a gauche
		code = code << 1;

		if(RC5_IN) code = code | 0b00000001;
		RC5WaitBit(!RC5_IN);
		Delay1332US();
		NbBits--;
	}

	//Reception de la commande
	NbBits = 6;
	while(NbBits)
	{
		commande = commande <<1;

		if(RC5_IN) commande = commande | 0b00000001;
		RC5WaitBit(!RC5_IN);
		Delay1332US();
		NbBits--;
	}

	Delay1332US();
	Delay1332US();
}

void RC5WaitBit(unsigned char code)
{
	//cette fonction va permettre d'attendre le prochain front

	unsigned char i,j;

	//boucle, qui permet d'viter de bloquer le pic en cas de mauvaise reception
	for(i=0x2D; i!=0; i--)
	{
		//tempo	
		for(j=0x20;j!=0;j--) {}
		
		//on vient lire le port IR
		if(code == RC5_IN) return;	//on a trouv un changement de valeur
	}
}


void RC5Init(void)
{
	//Cette fonction permet de faire l'initialisation rc5

	//On place le port rc5 en entre
	RC5_TRIS = RC5_TRIS | RC5_MSQ;

	//reset
	code = 0;
	commande = 0;

	//on vient lire le code auto-appris
	code_telecommande = EEPromLecture(EEPROM_CODE_RC5);
}

void RC5Traitement(void)
{
	//Cette fonction va tre appelle aprs avoir reu un 
	//code rc5. Elle permet d'effectuer l'action slectionne
	//a la tlcommande 

	unsigned char volplus[8] = "VOLPLUS";
	unsigned char volmoins[8] = "VOLMOIN";
	unsigned char msg[8] = "CODE OK";
	unsigned char msg2[8] = "CODENOK";

	//On regarde tout d'abord le code appareil
	if (code == 0x14)
	{
		//On fait le traitement
		switch(commande)
		{
			case RC5_VOLUME_PLUS: 
			{
				AFFReset();
				AFFText(volplus);
				break;
			}
			case RC5_VOLUME_MOINS:
			{
				AFFReset();
				AFFText(volmoins);
				break;
			}
			default:
			{
				AFFReset();
				AFFText(msg);
				break;
			}
			//case RC5_POWER: CDMarcheArret(); break;
		}

		//On reset le code/commande
		code = 0;
		commande = 0;
	}
}

void RC5AutoApprentissage(void)
{
	//Cette fonction sert  faire l'auto-apprentissage de la tlcommande.
	//Elle est dclenche par un appui long sur un des boutons

	unsigned char msg[8] = "AUTOAPP";
	unsigned char msg2[8] = "APP OK ";

	//On affiche un message a l'cran
	AFFText(msg);

	//On reset code et commande
	code = 0;
	commande = 0;
	
	//On attend qu'un code rc5 soit entr
	while (code == 0) { }

	//On enregistre le code recu
	EEPromEcriture(EEPROM_CODE_RC5, 0x20);

	//On affiche un message a l'ecran
	AFFReset();
	AFFText(msg2);

	//On attend
	DelayS(1);

	//On efface le texte
	AFFReset();
}
